#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"

enum board_e {
	BOARD_IDLE1 = 0,
	BOARD_IDLE2,
	BOARD_PLACE,
	BOARD_FIDGET,
	BOARD_DRAW,
	BOARD_WORLD,
	BOARD_GROUND
};

LINK_ENTITY_TO_CLASS( weapon_board, CBoard );

void CBoard::Spawn( )
{
	Precache( );
	pev->classname = MAKE_STRING( "weapon_board" ); // hack to allow for old names

	SET_MODEL(ENT(pev), "models/p_board.mdl");
	pev->scale = 4;
	m_iId = WEAPON_BOARD;

	m_iDefaultAmmo = BOARD_DEFAULT_GIVE;

	m_fPlacing = FALSE;

	FallInit();// get ready to fall down.
}

void CBoard::Precache( void )
{

	// Dunno if these are the right models
	PRECACHE_MODEL("models/v_board.mdl");
	PRECACHE_MODEL("models/p_board.mdl");
	UTIL_PrecacheOther( "monster_board" );

	// Add sounds later
	// PRECACHE_SOUND("items/9mmclip1.wav");

	// this is to hook your client-side event
	m_usBoardPlace = PRECACHE_EVENT( 1, "events/board1.sc" );
	m_usBoardNail  = PRECACHE_EVENT( 1, "events/board2.sc" );
}

int CBoard::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "board";
	p->iMaxAmmo1 = BOARD_MAX_CARRY;
	p->pszAmmo2 = NULL; // no secondary ammo
	p->iMaxAmmo2 = -1;
	p->iMaxClip = -1; // means that there is no clip
	p->iSlot = 0;
	p->iPosition = 1;
	p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
	p->iId = m_iId = WEAPON_BOARD;
	p->iWeight = BOARD_WEIGHT;

	return 1;
}

BOOL CBoard::Deploy( )
{
	return DefaultDeploy( "models/v_board.mdl", "models/p_board.mdl", BOARD_IDLE1, "tripmine" );
}

void CBoard::Holster( int skiplocal /* = 0 */ )
{
	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;

	if (!m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
	{
		// out of boards
		if(m_fPlacing)
		{
			m_fPlacing = FALSE;
			m_pPlayer->EnableControl(TRUE);
		}
		m_pPlayer->pev->weapons &= ~(1<<WEAPON_BOARD);
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}

	SendWeaponAnim( BOARD_IDLE1 );
}

void CBoard::PrimaryAttack( void )
{
	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		return;

	UTIL_MakeVectors( m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle );
	Vector vecSrc	 = m_pPlayer->GetGunPosition( );
	Vector vecAiming = gpGlobals->v_forward;
	Vector vecCenter = vecSrc + (vecAiming * 64);
	TraceResult tr;

	CBaseEntity *pEntity = NULL;
	pEntity = UTIL_FindEntityInSphere( pEntity, vecCenter, 20);
	
	if ( pEntity && (pEntity->pev->flags & FL_WINDOW) )
	{
		ALERT ( at_console, "Found Window\n" );
		pEntity->pev->solid = SOLID_BSP;
		UTIL_SetOrigin( pEntity->pev, pEntity->pev->origin );

		UTIL_TraceLine( vecSrc, vecSrc + vecAiming * 128, ignore_monsters, ENT( m_pPlayer->pev ), &tr );
		if (tr.flFraction < 1.0)
		{
			ALERT ( at_console, "Hit Window\n" );
			CBaseEntity *pEntity = CBaseEntity::Instance( tr.pHit );
			if ( pEntity && (pEntity->pev->flags & FL_WINDOW) )
			{
				SendWeaponAnim( BOARD_PLACE );
				Vector angles = UTIL_VecToAngles(tr.vecPlaneNormal);
				
				// Create Board on Window
				CBaseEntity *pEnt = CBaseEntity::Create( "monster_board", tr.vecEndPos + tr.vecPlaneNormal, angles, NULL );
				pEnt->pev->angles.x += 10 - RANDOM_LONG(0,20);
				pEnt->pev->absmin = Vector(	tr.vecEndPos.x + tr.vecPlaneNormal.x - 20, 
											tr.vecEndPos.y + tr.vecPlaneNormal.y - 20,
											tr.vecEndPos.z + tr.vecPlaneNormal.z - 20);
				pEnt->pev->absmax = Vector(	tr.vecEndPos.x + tr.vecPlaneNormal.x + 20, 
											tr.vecEndPos.y + tr.vecPlaneNormal.y + 20,
											tr.vecEndPos.z + tr.vecPlaneNormal.z + 20);
				

				m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

				// player "shoot" animation
				m_pPlayer->SetAnimation( PLAYER_ATTACK1 );
				m_pPlayer->EnableControl(FALSE);
				m_fPlacing = TRUE;
				m_flTimeWeaponIdle = m_flNextPrimaryAttack = m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 3;
			}
		}
		pEntity->pev->solid = SOLID_NOT;
		UTIL_SetOrigin( pEntity->pev, pEntity->pev->origin );
	}

	m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 0.5;
	m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 3;
}

void CBoard::WeaponIdle( void )
{
	if ( m_flTimeWeaponIdle > UTIL_WeaponTimeBase() )
		return;

	if ( m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 0 ) {
		SendWeaponAnim( BOARD_DRAW );
	}
	else
	{
		RetireWeapon();
		if(m_fPlacing)
		{
			m_fPlacing = FALSE;
			m_pPlayer->EnableControl(TRUE);
		}
		return;
	}

	if(m_fPlacing)
	{
		m_fPlacing = FALSE;
		m_pPlayer->EnableControl(TRUE);
		return;
	}

	int iAnim;
	float flRand = UTIL_SharedRandomFloat( m_pPlayer->random_seed, 0, 1 );
	if (flRand <= 0.25)
	{
		iAnim = BOARD_IDLE1;
		m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 90.0 / 30.0;
	}
	else if (flRand <= 0.75)
	{
		iAnim = BOARD_IDLE2;
		m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 60.0 / 30.0;
	}
	else
	{
		iAnim = BOARD_FIDGET;
		m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 100.0 / 30.0;
	}

	SendWeaponAnim( iAnim );
}