/***
*
*	Copyright (c) 1996-2002, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   This source code contains proprietary and confidential information of
*   Valve LLC and its suppliers.  Access to this code is restricted to
*   persons who have executed a written SDK license with Valve.  Any access,
*   use or distribution of this code by or to any unlicensed person is illegal.
*
****/
//=========================================================
// Resident Evil: Zombie
//=========================================================

#include	"extdll.h"
#include	"util.h"
#include	"cbase.h"
#include	"monsters.h"
#include	"schedule.h"
#include    "animation.h"
#include	"player.h"

//=========================================================
// monster-specific tasks
//=========================================================
enum
{
	TASK_GET_ROUTE_TO_WINDOW = LAST_COMMON_TASK + 1,
	TASK_ZOMBIE_REACH,
};

//=========================================================
// monster-specific schedule types
//=========================================================
enum
{
	SCHED_ZOMBIE_WALK = LAST_COMMON_SCHEDULE + 1,
	SCHED_ZOMBIE_FALL,
	SCHED_ZOMBIE_CRAWL,
	SCHED_ZOMBIE_GETUP,
	SCHED_ZOMBIE_BITE,
	SCHED_ZOMBIE_REACH,
	SCHED_ZOMBIE_EAT_REMAINS,
	SCHED_ZOMBIE_STEP_BACK,
	SCHED_FIND_DIFF_PATH,
};

//=========================================================
// Monster's Anim Events Go Here
//=========================================================
#define ZOMBIE_AE_BITE			( 1 )
#define ZOMBIE_AE_REACH			( 2 )
#define ZOMBIE_AE_FALLDOWN		( 3 )
#define ZOMBIE_AE_STEPBACK		( 4 )
#define ZOMBIE_AE_GETUP			( 5 )
#define ZOMBIE_AE_CLAW			( 6 )
#define ZOMBIE_AE_HEADBUST		( 7 )


class CZombie : public CBaseMonster
{
public:
	void Spawn( void );
	void Precache( void );
	void SetYawSpeed( void );
	int  Classify ( void );
	void HandleAnimEvent( MonsterEvent_t *pEvent );
	void RunTask (Task_t *pTask);
	void StartTask ( Task_t *pTask );

	void DeathSound( void );
	void PainSound( void );

	void SetActivity ( Activity NewActivity );

	Schedule_t	*GetSchedule( void );
	Schedule_t  *GetScheduleOfType ( int Type );
	CUSTOM_SCHEDULES;

	BOOL CheckMeleeAttack1 ( float flDot, float flDist );
	CBaseEntity *Bite();

	int TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType );
};

LINK_ENTITY_TO_CLASS( monster_zombie, CZombie );

//=========================================================
// Classify - indicates this monster's place in the 
// relationship table.
//=========================================================
int	CZombie :: Classify ( void )
{
	return	CLASS_ZOMBIE_HUMAN;
}

//=========================================================
// SetYawSpeed - allows each sequence to have a different
// turn rate associated with it.
//=========================================================
void CZombie :: SetYawSpeed ( void )
{
	int ys;

	ys = 150;

#if 0
	switch ( m_Activity )
	{
	}
#endif

	pev->yaw_speed = ys;
}

int CZombie :: TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType )
{
	// Take 30% damage from bullets
	if ( bitsDamageType == DMG_BULLET )
	{
		Vector vecDir = pev->origin - (pevInflictor->absmin + pevInflictor->absmax) * 0.5;
		vecDir = vecDir.Normalize();
		float flForce = DamageForce( flDamage );
		pev->velocity = pev->velocity + vecDir * flForce;
		flDamage *= 0.3;
	}

	return CBaseMonster::TakeDamage( pevInflictor, pevAttacker, flDamage, bitsDamageType );
}


//=========================================================
// HandleAnimEvent - catches the monster-specific messages
// that occur when tagged animation frames are played.
//=========================================================
void CZombie :: HandleAnimEvent( MonsterEvent_t *pEvent )
{
	CBaseEntity *pHurt;
	switch( pEvent->event )
	{
	case 1:
		pHurt = Bite();
		if ( pHurt )
		{
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/zm_feed.wav", 1, ATTN_NORM );
			UTIL_MakeVectors( pev->angles );
			if(pHurt->IsAlive())
			{
				if(!FStrEq(STRING(pHurt->pev->classname), "monster_board"))
				{
					pHurt->pev->punchangle.x = 15;
					pHurt->pev->velocity = pHurt->pev->velocity + gpGlobals->v_forward * 100 + gpGlobals->v_up * 50;
				}
				pHurt->TakeDamage( pev, pev, RANDOM_FLOAT(25,40), (DMG_SLASH | DMG_ALWAYSGIB) );
			}
		}
		break;
	case 2:
		pHurt = Bite();
		if ( pHurt && !FStrEq(STRING(pHurt->pev->classname), "monster_board") )
		{
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/zm_feed.wav", 1, ATTN_NORM );
			UTIL_MakeVectors( pev->angles );
			if(pHurt->IsAlive())
			{
				if(!FStrEq(STRING(pHurt->pev->classname), "monster_board"))
				{
					pHurt->pev->punchangle.x = 15;
					pHurt->pev->velocity = pHurt->pev->velocity + gpGlobals->v_forward * 100 + gpGlobals->v_up * 50;
				}
				pHurt->TakeDamage( pev, pev, RANDOM_FLOAT(25,40), (DMG_SLASH | DMG_ALWAYSGIB) );
				if(RANDOM_FLOAT(0.0,1.0) <= 0.75 && pHurt->IsAlive() && pHurt->IsPlayer())
				{
					CBasePlayer *pVictim = (CBasePlayer *)CBaseEntity::Instance(pHurt->pev);
					if(pVictim != NULL)
					{
						pVictim->pev->velocity = g_vecZero;
						pVictim->EnableControl(FALSE);
						pVictim->m_fGrabbed = TRUE;
						pVictim->m_flFreezeTime = gpGlobals->time + 2.5;
					}
				}
			}
		}
		break;
	default:
		CBaseMonster::HandleAnimEvent( pEvent );
		break;
	}
}

//=========================================================
// Spawn
//=========================================================
void CZombie :: Spawn()
{
	Precache( );

	switch(RANDOM_LONG(0,6))
	{
	case 0:	SET_MODEL(ENT(pev), "models/zombie.mdl"); break;
	case 1:	SET_MODEL(ENT(pev), "models/zombie1.mdl"); break;
	case 2:	SET_MODEL(ENT(pev), "models/zombie2.mdl"); break;
	case 3:	SET_MODEL(ENT(pev), "models/zombie3.mdl"); break;
	case 4:	SET_MODEL(ENT(pev), "models/zombie4.mdl"); break;
	case 5:	SET_MODEL(ENT(pev), "models/zombie5.mdl"); break;
	case 6:	SET_MODEL(ENT(pev), "models/zombie6.mdl"); break;
	}
	UTIL_SetSize( pev, VEC_HUMAN_HULL_MIN, VEC_HUMAN_HULL_MAX );

	pev->solid			= SOLID_SLIDEBOX;
	pev->movetype		= MOVETYPE_STEP;
	m_bloodColor		= BLOOD_COLOR_RED;
	pev->health			= gSkillData.zombieHealth;
	pev->view_ofs		= VEC_VIEW;// position of the eyes relative to monster's origin.
	m_flFieldOfView		= 0.4;// indicates the width of this monster's forward view cone ( as a dotproduct result )
	m_MonsterState		= MONSTERSTATE_IDLE;
		
	MonsterInit();
}

//=========================================================
// Precache - precaches all resources this monster needs
//=========================================================
void CZombie :: Precache()
{
	PRECACHE_MODEL("models/zombie.mdl");
	PRECACHE_MODEL("models/zombie1.mdl");
	PRECACHE_MODEL("models/zombie2.mdl");
	PRECACHE_MODEL("models/zombie3.mdl");
	PRECACHE_MODEL("models/zombie4.mdl");
	PRECACHE_MODEL("models/zombie5.mdl");
	PRECACHE_MODEL("models/zombie6.mdl");

	// Zombie Scream Sounds
	PRECACHE_SOUND("Monsters/Zombie/zm_scream1.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_scream2.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_scream3.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_scream4.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_scream5.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_scream6.wav");

	// Zombie Walking Sounds
	PRECACHE_SOUND("Monsters/Zombie/zm_walk1.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_walk2.wav");

	// Zombie Pain Sounds
	PRECACHE_SOUND("Monsters/Zombie/zm_pain1.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_pain2.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_pain3.wav");

	// Miscellaneous Zombie Sounds
	PRECACHE_SOUND("Monsters/Zombie/zm_barf.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_feed.wav");
	PRECACHE_SOUND("Monsters/Zombie/zm_fall.wav");
}

void CZombie :: StartTask ( Task_t *pTask )
{
	m_iTaskStatus = TASKSTATUS_RUNNING;

	switch ( pTask->iTask )
	{
	case TASK_GET_ROUTE_TO_WINDOW:
		{
			UTIL_MakeVectors( pev->v_angle + pev->punchangle );
			Vector vecSrc	 = GetGunPosition( );
			Vector vecAiming = gpGlobals->v_forward;
			Vector vecCenter = vecSrc + (vecAiming * 32);
			TraceResult tr;

			CBaseEntity *pEntity = NULL;
			pEntity = UTIL_FindEntityInSphere( pEntity, vecCenter, 40);
			ALERT(at_aiconsole, "Trying to find window\n");
			
			if ( pEntity && (pEntity->pev->flags & FL_WINDOW) )
			{
				ALERT(at_aiconsole, "Found window\n");
				if ( BuildRoute ( pEntity->pev->origin, bits_MF_TO_LOCATION, pEntity ) )
				{
					TaskComplete();
				}
				else
				{
					TaskFail();
				}
			}
			TaskFail();
			break;
		}
	default: 
		CBaseMonster :: StartTask( pTask );
		break;
	}
}

void CZombie :: RunTask (Task_t *pTask)
{
	switch( pTask->iTask )
	{
	case TASK_GET_ROUTE_TO_WINDOW:
	case TASK_ZOMBIE_REACH:
		if(m_fSequenceFinished)
		{
			TaskComplete();
			SetTouch( NULL );
			m_IdealActivity = ACT_IDLE;
		}
		break;
	default:
		CBaseMonster::RunTask(pTask);
	}
}

//=========================================================
// PainSound
//=========================================================
void CZombie :: PainSound ( void )
{
		switch ( RANDOM_LONG(0,4) )
		{
		case 0:	
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/zm_pain1.wav", 1, ATTN_NORM );	
			break;
		case 1:
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/zm_pain2.wav", 1, ATTN_NORM );	
			break;
		case 2:
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/am_pain3.wav", 1, ATTN_NORM );	
			break;
		}
}

//=========================================================
// DeathSound 
//=========================================================
void CZombie :: DeathSound ( void )
{
	switch ( RANDOM_LONG(0,2) )
	{
	case 0:	
		EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/zm_die1.wav", 1, ATTN_IDLE );	
		break;
	case 1:
		EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/zm_die2.wav", 1, ATTN_IDLE );	
		break;
	case 2:
		EMIT_SOUND( ENT(pev), CHAN_VOICE, "zombie/zm_die3.wav", 1, ATTN_IDLE );	
		break;
	}
}

Task_t tlZombieFail[] =
{
	{ TASK_STOP_MOVING,			0							},
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE				},
	{ TASK_WAIT,				(float)2					},
	{ TASK_SET_SCHEDULE,		(float)SCHED_CHASE_ENEMY	},
};

Schedule_t slZombieFail[] = 
{
	{
		tlZombieFail,
		ARRAYSIZE ( tlZombieFail ),
		bits_COND_CAN_MELEE_ATTACK1 |
		bits_COND_LIGHT_DAMAGE |
		bits_COND_HEAVY_DAMAGE,
		0,
		"Zombie Fail"
	},
};

Task_t tlZombieBite[] = 
{
	{ TASK_STOP_MOVING,			0					},
	{ TASK_FACE_ENEMY,			(float)0			},
	//{ TASK_PLAY_SEQUENCE,		(float)ACT_MELEE_ATTACK1 	},
	{ TASK_MELEE_ATTACK1,		(float)0	},
};

Schedule_t slZombieBite[] = 
{
	{
		tlZombieBite,
		ARRAYSIZE ( tlZombieBite ),
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE,
		0,
		"Zombie Bite"
	},
};
/*
Task_t tlZombieReach[] =
{
	{ TASK_STOP_MOVING,				0			},
	{ TASK_GET_ROUTE_TO_WINDOW,		(float)0	},
};
*/

Task_t tlZombieEatDead[] =
{
	{ TASK_STOP_MOVING,					0				},
	{ TASK_FACE_ENEMY,					(float)0		},
	{ TASK_WAIT,						(float)1.5		},
	{ TASK_GET_PATH_TO_ENEMY_CORPSE,	(float)0		},
	{ TASK_WALK_PATH,					(float)0		},
	{ TASK_WAIT_FOR_MOVEMENT,			(float)0		},
	{ TASK_FACE_ENEMY,					(float)0		},
	{ TASK_PLAY_SEQUENCE,				(float)ACT_EAT	},
};

Schedule_t slZombieEatDead[] = 
{
	{
		tlZombieEatDead,
		ARRAYSIZE ( tlZombieEatDead ),
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE,
		0,
		"Zombie Eat"
	},
};

Task_t tlZombieFall[] =
{
	{ TASK_STOP_MOVING,			0					},
	{ TASK_PLAY_SEQUENCE,		(float)ACT_FALL		},
};

Schedule_t slZombieFall[] =
{
	{
		tlZombieFall,
		ARRAYSIZE ( tlZombieFall ),
		0,
		0,
		"Zombie Fall"
	},
};

Task_t tlZombieStepBack[] = 
{
	{ TASK_STOP_MOVING,			0							},
	{ TASK_PLAY_SEQUENCE,		(float)ACT_FLINCH_CHEST 	},
	{ TASK_STEP_BACK,			(float)2					},
};

Schedule_t slZombieStepBack[] =
{
	{
		tlZombieStepBack,
		ARRAYSIZE ( tlZombieStepBack ),
		0,
		0,
		"Zombie Step Back"
	},
};
/*
Task_t tlZombieBlocked[] = 
{
	{ TASK_STOP_MOVING,			0								},
	{ TASK_SET_FAIL_SCHEDULE,	(float)SCHED_FIND_DIFF_PATH		},
	{ TASK_GET_ROUTE_TO_WINDOW,	(float)0						},
	{ TASK_ZOMBIE_REACH,		(float)0						},
};

Schedule_t slZombieBlocked[] =
{
	{
		tlZombieBlocked,
		ARRAYSIZE ( tlZombieBlocked ),
		0,
		0,
		"Zombie Blocked"
	},
};
*/

DEFINE_CUSTOM_SCHEDULES( CZombie )
{
	slZombieFail,
	slZombieBite,
	slZombieFall,
	slZombieEatDead,
	slZombieStepBack,
};

IMPLEMENT_CUSTOM_SCHEDULES( CZombie, CBaseMonster );

void CZombie :: SetActivity ( Activity NewActivity )
{
	int	iSequence = ACTIVITY_NOT_AVAILABLE;
	void *pmodel = GET_MODEL_PTR( ENT(pev) );

	switch ( NewActivity)
	{
	case ACT_BITE:
		break;
	default:
		CBaseMonster::SetActivity(NewActivity);
		break;
	}
}

Schedule_t *CZombie :: GetSchedule( void )
{
	switch	( m_MonsterState )
	{
	case MONSTERSTATE_COMBAT:
		{
			if ( HasConditions( bits_COND_ENEMY_DEAD ) )
			{
				return GetScheduleOfType ( SCHED_ZOMBIE_EAT_REMAINS );
			}
			else if ( HasConditions(bits_COND_NEW_ENEMY) )
			{
				if ( HasConditions ( bits_COND_CAN_MELEE_ATTACK1 ) )
				{
					return GetScheduleOfType ( SCHED_ZOMBIE_BITE );
				}
				else
				{
					ALERT(at_aiconsole, "ZOMBIE: Chase Enemy!\n");
					return GetScheduleOfType ( SCHED_CHASE_ENEMY );
				}
			}
			else if ( HasConditions ( bits_COND_CAN_MELEE_ATTACK1 ) )
			{
				return GetScheduleOfType ( SCHED_MELEE_ATTACK1 );
			}
			else if ( HasConditions( bits_COND_LIGHT_DAMAGE ) )
			{
				int iPercent = RANDOM_LONG(0,99);

				if ( iPercent <= 90 && m_hEnemy != NULL )
				{
					return GetScheduleOfType( SCHED_CHASE_ENEMY );
				}
				else
				{
					return GetScheduleOfType( SCHED_ZOMBIE_FALL );
				}
			}
			else if ( HasConditions ( bits_COND_HEAVY_DAMAGE ) )
			{
				int iPercent = RANDOM_LONG(0,99);

				if ( iPercent <= 80 && m_hEnemy != NULL )
				{
					return GetScheduleOfType( SCHED_ZOMBIE_STEP_BACK );
				}
				else
				{
					return GetScheduleOfType ( SCHED_ZOMBIE_FALL );
				}
			}
		}
	}
	return CBaseMonster :: GetSchedule();
}

Schedule_t* CZombie :: GetScheduleOfType ( int Type ) 
{
	switch	( Type )
	{
	case SCHED_ZOMBIE_FALL:
		{
			return &slZombieFall[ 0 ];
		}
	case SCHED_ZOMBIE_BITE:
		{
			return &slZombieBite[ 0 ];
		}
	case SCHED_MELEE_ATTACK1:
		{
			return &slZombieBite[ 0 ];
		}
	case SCHED_ZOMBIE_EAT_REMAINS:
		{
			return &slZombieEatDead[ 0 ];
		}
	case SCHED_FAIL:
		{
			return &slZombieFail[ 0 ];
		}
	case SCHED_ZOMBIE_STEP_BACK:
		{
			return &slZombieStepBack[ 0 ];
		}
	}
	return CBaseMonster :: GetScheduleOfType(Type);
}

BOOL CZombie :: CheckMeleeAttack1 ( float flDot, float flDist )
{
	if ( flDist <= 56 && flDot >= 0.7 && m_hEnemy != NULL && FBitSet(m_hEnemy->pev->flags, FL_ONGROUND) )
	{
		return TRUE;
	}
	return FALSE;
}

CBaseEntity *CZombie :: Bite( void )
{
	TraceResult tr;

	UTIL_MakeVectors( pev->angles );
	Vector vecStart = pev->origin;
	vecStart.z += pev->size.z * 0.5;
	Vector vecEnd = vecStart + (gpGlobals->v_forward * 64);

	UTIL_TraceHull( vecStart, vecEnd, dont_ignore_monsters, head_hull, ENT(pev), &tr );
	
	if ( tr.pHit )
	{
		CBaseEntity *pEntity = CBaseEntity::Instance( tr.pHit );
		return pEntity;
	}

	return NULL;
}